import React from 'react';
import { matchMedia } from '@shopify/jest-dom-mocks';
import faker from 'faker';
import { mountWithContext } from '../../test-utilities';
import { UnstyledLink } from '../Link';
import { Spinner } from '../Spinner';
import { Form } from '../Form';
import { Button } from './Button';
var defaultProps = {
  children: 'Go to next step'
};
describe('<Button />', function () {
  beforeEach(function () {
    matchMedia.mock();
  });
  afterEach(function () {
    matchMedia.restore();
  });
  describe('`<button />`', function () {
    describe('children', function () {
      it('renders the children into the button', function () {
        var text = 'Go to shipping';
        var button = mountWithContext( /*#__PURE__*/React.createElement(Button, null, text));
        expect(button).toContainReactText(text);
      });
      it('renders the children into the link', function () {
        var text = 'Go to shipping';
        var linkButton = mountWithContext( /*#__PURE__*/React.createElement(Button, {
          to: "/shipping"
        }, text));
        expect(linkButton).toContainReactText(text);
      });
    });
    describe('onPress()', function () {
      it('is called when button is pressed', function () {
        var onPressSpy = jest.fn();
        var button = mountWithContext( /*#__PURE__*/React.createElement(Button, Object.assign({}, defaultProps, {
          onPress: onPressSpy
        })));
        button.find('button').trigger('onClick');
        expect(onPressSpy).toHaveBeenCalledTimes(1);
      });
    });
  });
  describe('`<UnstyledLink />`', function () {
    it('renders when a url is provided', function () {
      var url = '/shipping';
      var button = mountWithContext( /*#__PURE__*/React.createElement(Button, Object.assign({}, defaultProps, {
        to: url
      })));
      expect(button).toContainReactComponent(UnstyledLink, {
        to: url
      });
    });
    it('renders a button when disabled', function () {
      var button = mountWithContext( /*#__PURE__*/React.createElement(Button, Object.assign({}, defaultProps, {
        disabled: true,
        to: "/shipping"
      })));
      expect(button).not.toContainReactComponent(UnstyledLink);
      expect(button).toContainReactComponent('button', {
        disabled: true
      });
    });
  });
  describe('submit', function () {
    it('renders a type button when not present', function () {
      var button = mountWithContext( /*#__PURE__*/React.createElement(Button, defaultProps));
      expect(button).toContainReactComponent('button', {
        type: 'button'
      });
    });
    it('renders a type submit when present', function () {
      var button = mountWithContext( /*#__PURE__*/React.createElement(Button, Object.assign({}, defaultProps, {
        submit: true
      })));
      expect(button).toContainReactComponent('button', {
        type: 'submit'
      });
    });
    it('renders a form id when rendered in a nested form', function () {
      var testId = faker.random.word();
      var button = mountWithContext( /*#__PURE__*/React.createElement(Form, {
        onSubmit: noop
      }, /*#__PURE__*/React.createElement(Form, {
        id: testId,
        onSubmit: noop
      }, /*#__PURE__*/React.createElement(Button, Object.assign({}, defaultProps, {
        submit: true
      })))));
      expect(button).toContainReactComponent('button', {
        type: 'submit',
        form: testId
      });
    });
    it('does not render a form id when rendered in a non-nested form', function () {
      var button = mountWithContext( /*#__PURE__*/React.createElement(Form, {
        onSubmit: noop
      }, /*#__PURE__*/React.createElement(Button, Object.assign({}, defaultProps, {
        submit: true
      }))));
      expect(button).toContainReactComponent('button', {
        type: 'submit',
        form: undefined
      });
    });
  });
  describe('loading', function () {
    it('disables button when loading', function () {
      var button = mountWithContext( /*#__PURE__*/React.createElement(Button, Object.assign({}, defaultProps, {
        loading: true
      })));
      expect(button).toContainReactComponent('button', {
        disabled: true
      });
    });
    it('indicates that the button is getting updated', function () {
      var button = mountWithContext( /*#__PURE__*/React.createElement(Button, Object.assign({}, defaultProps, {
        loading: true
      })));
      expect(button).toContainReactComponent('button', {
        'aria-busy': true
      });
      var unstyledLink = mountWithContext( /*#__PURE__*/React.createElement(Button, Object.assign({}, defaultProps, {
        loading: true,
        to: "/shipping"
      })));
      expect(unstyledLink).toContainReactComponent(UnstyledLink, {
        ariaBusy: true
      });
    });
    it('does not indicate that the button is getting updated when not set', function () {
      var button = mountWithContext( /*#__PURE__*/React.createElement(Button, defaultProps));
      expect(button).toContainReactComponent('button', {
        'aria-busy': undefined
      });
    });
    it('sets the right priority of announcement for screen readers', function () {
      var button = mountWithContext( /*#__PURE__*/React.createElement(Button, Object.assign({}, defaultProps, {
        loading: true
      })));
      expect(button).toContainReactComponent('button', {
        'aria-live': 'assertive'
      });
      var unstyledLink = mountWithContext( /*#__PURE__*/React.createElement(Button, Object.assign({}, defaultProps, {
        loading: true,
        to: "/shipping"
      })));
      expect(unstyledLink).toContainReactComponent(UnstyledLink, {
        ariaLive: 'assertive'
      });
    });
    it('renders a <Spinner> when loading', function () {
      matchMedia.setMedia(function () {
        return {
          matches: false
        };
      });
      var button = mountWithContext( /*#__PURE__*/React.createElement(Button, Object.assign({}, defaultProps, {
        loading: true
      })));
      expect(button).toContainReactComponent(Spinner);
    });
    it('renders the loadingLabel as the Spinner children when provided', function () {
      var button = mountWithContext( /*#__PURE__*/React.createElement(Button, Object.assign({}, defaultProps, {
        loadingLabel: "Processing",
        loading: true
      })));
      expect(button).toContainReactComponent(Spinner, {
        accessibilityLabel: 'Processing'
      });
    });
    it('adds the Spinner class when user does not prefers reduced motion', function () {
      matchMedia.setMedia(function () {
        return {
          matches: false
        };
      });
      var button = mountWithContext( /*#__PURE__*/React.createElement(Button, Object.assign({}, defaultProps, {
        loadingLabel: "Processing",
        loading: true
      })));
      expect(button).toContainReactComponent('span', {
        className: 'LoadingContent Spinner'
      });
    });
    it('removes the Spinner class when user prefers reduced motion', function () {
      matchMedia.setMedia(function () {
        return {
          matches: true
        };
      });
      var button = mountWithContext( /*#__PURE__*/React.createElement(Button, Object.assign({}, defaultProps, {
        loadingLabel: "Processing",
        loading: true
      })));
      expect(button).not.toContainReactComponent('span', {
        className: 'LoadingContent Spinner'
      });
    });
  });
  describe('accessibilityLabel', function () {
    it('renders an accessible label when provided', function () {
      var content = 'Accessible label';
      var button = mountWithContext( /*#__PURE__*/React.createElement(Button, Object.assign({}, defaultProps, {
        accessibilityLabel: content
      })));
      expect(button).toContainReactComponent('button', {
        'aria-label': content
      });
      var unstyledLink = mountWithContext( /*#__PURE__*/React.createElement(Button, Object.assign({}, defaultProps, {
        accessibilityLabel: content,
        to: "/shipping"
      })));
      expect(unstyledLink).toContainReactComponent(UnstyledLink, {
        ariaLabel: content
      });
    });
  });
});

function noop() {}