import React from 'react';
import faker from 'faker';
import { mountWithContext } from '../../test-utilities';
import { InlineError } from '../InlineError';
import { Checkbox } from './Checkbox';
var defaultProps = {
  children: 'Save this information for next time'
};
describe('<Checkbox />', function () {
  it('renders an un-checked checkbox and label', function () {
    var checkbox = mountWithContext( /*#__PURE__*/React.createElement(Checkbox, defaultProps));
    expect(checkbox).toContainReactComponent('input', {
      checked: false,
      type: 'checkbox'
    });
    expect(checkbox.find('label')).toContainReactText('Save this information for next time');
  });
  it('renders a checked checkbox when the checked prop is provided and set to true', function () {
    var checkbox = mountWithContext( /*#__PURE__*/React.createElement(Checkbox, Object.assign({}, defaultProps, {
      checked: true
    })));
    expect(checkbox).toContainReactComponent('input', {
      checked: true,
      type: 'checkbox'
    });
  });
  it('renders a checked checkbox when the value prop is provided and set to true', function () {
    var checkbox = mountWithContext( /*#__PURE__*/React.createElement(Checkbox, Object.assign({}, defaultProps, {
      value: true
    })));
    expect(checkbox).toContainReactComponent('input', {
      checked: true,
      type: 'checkbox'
    });
  });
  it('renders a disabled checkbox when the disabled prop is provided and set to true', function () {
    var checkbox = mountWithContext( /*#__PURE__*/React.createElement(Checkbox, Object.assign({}, defaultProps, {
      disabled: true
    })));
    expect(checkbox).toContainReactComponent('input', {
      disabled: true,
      type: 'checkbox'
    });
  });
  it('calls the onChange callback when the checkbox is checked and unchecked', function () {
    var onChangeSpy = jest.fn();
    var checkbox = mountWithContext( /*#__PURE__*/React.createElement(Checkbox, Object.assign({}, defaultProps, {
      onChange: onChangeSpy
    })));
    checkbox.find('input').trigger('onChange', {
      currentTarget: {
        checked: true
      }
    });
    expect(onChangeSpy).toHaveBeenCalledWith(true);
    checkbox.find('input').trigger('onChange', {
      currentTarget: {
        checked: false
      }
    });
    expect(onChangeSpy).toHaveBeenCalledWith(false);
  });
  describe('error', function () {
    it('renders an error message when the error prop is provided', function () {
      var error = 'Error message';
      var id = faker.random.alphaNumeric();
      var checkbox = mountWithContext( /*#__PURE__*/React.createElement(Checkbox, Object.assign({}, defaultProps, {
        id: id,
        error: error
      })));
      expect(checkbox).toContainReactComponent(InlineError, {
        controlID: id,
        children: error
      });
    });
    it('creates an ID automatically that associates an error with the input', function () {
      var _checkbox$find, _checkbox$find2;

      var error = 'Error message';
      var checkbox = mountWithContext( /*#__PURE__*/React.createElement(Checkbox, Object.assign({}, defaultProps, {
        error: error
      })));
      var id = (_checkbox$find = checkbox.find('input')) === null || _checkbox$find === void 0 ? void 0 : _checkbox$find.prop('id');
      expect(id).not.toBeUndefined();
      expect(checkbox).toContainReactComponent(InlineError, {
        controlID: (_checkbox$find2 = checkbox.find('input')) === null || _checkbox$find2 === void 0 ? void 0 : _checkbox$find2.prop('id'),
        children: error
      });
    });
  });
  describe('accessibilityLabel', function () {
    it('adds an aria-label to the label when the accessibilityLabel is set', function () {
      var accessibilityLabelContent = 'Accessibility content';
      var checkbox = mountWithContext( /*#__PURE__*/React.createElement(Checkbox, Object.assign({}, defaultProps, {
        accessibilityLabel: accessibilityLabelContent
      })));
      expect(checkbox).toContainReactComponent('label', {
        'aria-label': accessibilityLabelContent
      });
    });
    it('does not add aria-label to the label when accessibilityLabel is not set', function () {
      var accessibilityLabelContent = 'Accessibility content';
      var checkbox = mountWithContext( /*#__PURE__*/React.createElement(Checkbox, defaultProps, "Default content"));
      expect(checkbox).not.toContainReactComponent('label', {
        'aria-label': accessibilityLabelContent
      });
    });
  });
});