import React, { createContext, useContext, Children, useEffect, useCallback, useRef } from 'react';
import { classNames } from '@shopify/css-utilities';
import { Icon } from '../Icon';
import { InlineStack } from '../InlineStack';
import { createIdCreator, useId } from '../../utilities/id';
import styles from './Tabs.css';
var createId = createIdCreator('Tabs');
var TabsContext = /*#__PURE__*/createContext(null);
/**
 * Tabs are used when there’s multiple ways to display the same information,
 * like a map or list view. When a buyer needs to make a choice, use the
 * OptionList component instead as Buyers are more likely to choose the
 * non-default option. Don’t hide important information in tabs. Limit to two
 * of three tabs and keep labels short to optimize for mobile.
 */

export function Tabs(_ref) {
  var tabs = _ref.tabs,
      selected = _ref.selected,
      ariaLabel = _ref.ariaLabel,
      bordered = _ref.bordered,
      onChange = _ref.onChange,
      children = _ref.children;
  var id = useId(undefined, createId);
  var tabList = useRef(null);
  var lastIndex = tabs.length - 1;
  var childrenCount = Children.count(children);
  var handleKeyDown = useCallback(function (event) {
    var prev = selected - 1 < 0 ? lastIndex : selected - 1;
    var next = selected + 1 > lastIndex ? 0 : selected + 1;
    var handlerMap = new Map([['ArrowLeft', prev], ['Left', prev], ['ArrowRight', next], ['Right', next], ['Home', 0], ['End', lastIndex]]);
    var newSelected = handlerMap.get(event.key);

    if (newSelected != null) {
      event.preventDefault();
      onChange(newSelected);
    }
  }, [onChange, selected, lastIndex]);
  useEffect(function () {
    var current = tabList.current;
    if (current == null) return;
    current.addEventListener('keydown', handleKeyDown, false);
    return function () {
      current.removeEventListener('keydown', handleKeyDown, false);
    };
  }, [tabList, handleKeyDown]);
  useEffect(function () {
    if (childrenCount !== tabs.length) {
      // eslint-disable-next-line no-console
      console.warn("<Tabs /> has ".concat(tabs.length, " tabs and ").concat(childrenCount, " children. A mismatch will cause some content to be unavailable."));
    }
  }, [childrenCount, tabs.length]);
  var tabsMarkup = [];
  var panelsMarkup = [];
  Children.forEach(children, function (child, index) {
    var tab = tabs[index];

    if (tab != null) {
      var _ref2 = typeof tab === 'string' ? {
        label: tab,
        icon: undefined
      } : tab,
          label = _ref2.label,
          icon = _ref2.icon;

      tabsMarkup.push( /*#__PURE__*/React.createElement(Tab, Object.assign({
        label: label,
        icon: icon,
        selected: selected,
        onChange: onChange,
        id: id
      }, {
        index: index,
        key: label
      })));
    }

    panelsMarkup.push( /*#__PURE__*/React.createElement(TabPanel, {
      index: index
    }, child));
  });
  return /*#__PURE__*/React.createElement(TabsContext.Provider, {
    value: {
      id: id,
      selected: selected,
      bordered: bordered
    }
  }, /*#__PURE__*/React.createElement("div", {
    className: styles.Tabs
  }, /*#__PURE__*/React.createElement("div", {
    ref: tabList,
    role: "tablist",
    "aria-label": ariaLabel,
    className: styles.TabList
  }, tabsMarkup), panelsMarkup));
}

function Tab(_ref3) {
  var label = _ref3.label,
      icon = _ref3.icon,
      index = _ref3.index,
      selectedIndex = _ref3.selected,
      onChange = _ref3.onChange,
      id = _ref3.id;

  if (index == null) {
    throw new Error('`index` is missing');
  }

  var tabRef = useRef(null);
  var mountedRef = useRef(false);
  var selected = selectedIndex === index;
  var markup = icon == null ? label : /*#__PURE__*/React.createElement(InlineStack, {
    spacing: "extraTight",
    blockAlignment: "center"
  }, /*#__PURE__*/React.createElement(Icon, {
    source: icon,
    size: "large"
  }), /*#__PURE__*/React.createElement("span", null, label));
  useEffect(function () {
    if (tabRef.current && selected && mountedRef.current) tabRef.current.focus();else mountedRef.current = true;
  }, [tabRef, selected]);
  return /*#__PURE__*/React.createElement("button", {
    type: "button",
    role: "tab",
    tabIndex: selected ? 0 : -1,
    "aria-selected": selected,
    id: "".concat(id, "Tab").concat(index),
    "aria-controls": "".concat(id, "TabPanel").concat(index),
    onClick: function onClick() {
      return onChange(index);
    },
    className: classNames(styles.Tab, selected && styles.selected),
    ref: tabRef
  }, markup);
}

function TabPanel(_ref4) {
  var index = _ref4.index,
      children = _ref4.children;

  var _useTabs = useTabs(),
      selectedIndex = _useTabs.selected,
      id = _useTabs.id,
      bordered = _useTabs.bordered;

  var selected = selectedIndex === index;
  return /*#__PURE__*/React.createElement("div", {
    role: "tabpanel",
    id: "".concat(id, "TabPanel").concat(index),
    "aria-labelledby": "".concat(id, "Tab").concat(index),
    hidden: !selected,
    className: classNames(styles.TabPanel, bordered && styles.bordered)
  }, children);
}

function useTabs() {
  var context = useContext(TabsContext);

  if (context == null) {
    throw new Error('No tabs found in context');
  }

  return context;
}