import React, { useEffect } from 'react';
import { matchMedia } from '@shopify/jest-dom-mocks';
import { mountWithContext } from '../test-utilities';
import { isResponsiveValue, maybeResponsiveToArray, useResponsiveStyle, useResponsiveValue } from './responsiveValue';
describe('useResponsiveValue', function () {
  var spy = jest.fn();

  function Component(_ref) {
    var values = _ref.values;
    var value = useResponsiveValue(values);
    useEffect(function () {
      spy(value);
    });
    return null;
  }

  beforeEach(function () {
    matchMedia.mock();
  });
  afterEach(function () {
    matchMedia.restore();
    spy.mockReset();
  });
  it('returns undefined if no breakpoint matches', function () {
    var values = {
      small: 'small'
    };
    matchMedia.setMedia(function () {
      return {
        matches: false
      };
    });
    mountWithContext( /*#__PURE__*/React.createElement(Component, {
      values: values
    }));
    expect(spy.mock.calls).toMatchObject([[undefined]]);
  });
  it('returns base value if it maches base breakpoint', function () {
    var values = {
      base: 'base'
    };
    matchMedia.setMedia(function (query) {
      return {
        matches: query === '(min-width: 0px) and (max-width: 749px)'
      };
    });
    mountWithContext( /*#__PURE__*/React.createElement(Component, {
      values: values
    }));
    expect(spy.mock.calls).toMatchObject([[values.base]]);
  });
  it('returns small value if it maches small breakpoint', function () {
    var values = {
      small: 'small'
    };
    matchMedia.setMedia(function (query) {
      return {
        matches: query === '(min-width: 750px) and (max-width: 999px)'
      };
    });
    mountWithContext( /*#__PURE__*/React.createElement(Component, {
      values: values
    }));
    expect(spy.mock.calls).toMatchObject([[values.small]]);
  });
  it('returns medium value if it maches medium breakpoint', function () {
    var values = {
      medium: 'medium'
    };
    matchMedia.setMedia(function (query) {
      return {
        matches: query === '(min-width: 1000px) and (max-width: 1199px)'
      };
    });
    mountWithContext( /*#__PURE__*/React.createElement(Component, {
      values: values
    }));
    expect(spy.mock.calls).toMatchObject([[values.medium]]);
  });
  it('returns large value if it maches large breakpoint', function () {
    var values = {
      large: 'large'
    };
    matchMedia.setMedia(function (query) {
      return {
        matches: query === '(min-width: 1200px)'
      };
    });
    mountWithContext( /*#__PURE__*/React.createElement(Component, {
      values: values
    }));
    expect(spy.mock.calls).toMatchObject([[values.large]]);
  });
  it('returns closest value if no breakpoint matches', function () {
    var values = {
      small: 'small',
      large: 'large'
    };
    matchMedia.setMedia(function (query) {
      return {
        matches: query === '(min-width: 750px) and (max-width: 999px)'
      };
    });
    mountWithContext( /*#__PURE__*/React.createElement(Component, {
      values: values
    }));
    expect(spy.mock.calls).toMatchObject([[values.small]]);
  });
  it('returns closest non undefined value', function () {
    var values = {
      small: 'small',
      medium: null,
      large: undefined
    };
    matchMedia.setMedia(function (query) {
      return {
        matches: query === '(min-width: 1200px)'
      };
    });
    mountWithContext( /*#__PURE__*/React.createElement(Component, {
      values: values
    }));
    expect(spy.mock.calls).toMatchObject([[values.medium]]);
  });
});
describe('isResponsiveValue', function () {
  it('returns false when a value different of a responsive value is passed', function () {
    expect(isResponsiveValue(undefined)).toBe(false);
    expect(isResponsiveValue(null)).toBe(false);
    expect(isResponsiveValue('some value')).toBe(false);
    expect(isResponsiveValue({})).toBe(false);
    expect(isResponsiveValue({
      foo: 1,
      bar: 2
    })).toBe(false);
    expect(isResponsiveValue({
      base: 1,
      small: 2,
      medium: 3,
      foo: 4
    })).toBe(false);
  });
  it('returns true when responsive value is passed', function () {
    expect(isResponsiveValue({
      small: 'some value'
    })).toBe(true);
    expect(isResponsiveValue({
      base: 1,
      small: 2,
      medium: 3,
      large: 4
    })).toBe(true);
  });
});
describe('maybeResponsiveToArray', function () {
  it('returns an array containing the passed non-responsive value as the value for the "base" breakpoint', function () {
    expect(maybeResponsiveToArray('some value')).toStrictEqual([['base', 'some value']]);
  });
  it('returns an array containing the entries of the responsive value as an array', function () {
    expect(new Set(maybeResponsiveToArray({
      base: 1,
      small: 2,
      medium: 3,
      large: 4
    }))).toStrictEqual(new Set([['base', 1], ['small', 2], ['medium', 3], ['large', 4]]));
    expect(new Set(maybeResponsiveToArray({
      small: 2,
      medium: 3
    }))).toStrictEqual(new Set([['small', 2], ['medium', 3]]));
  });
});
describe('useResponsiveStyle', function () {
  var spy = jest.fn();

  function Component(_ref2) {
    var config = _ref2.config;
    var value = useResponsiveStyle(config);
    useEffect(function () {
      spy(value);
    });
    return null;
  }

  beforeEach(function () {
    matchMedia.mock();
    matchMedia.setMedia(function () {
      return {
        matches: true
      };
    });
  });
  afterEach(function () {
    matchMedia.restore();
    spy.mockReset();
  });
  it('returns an empty object if passed value is undefined', function () {
    mountWithContext( /*#__PURE__*/React.createElement(Component, {
      config: {
        color: {
          value: undefined
        }
      }
    }));
    expect(spy.mock.calls).toMatchObject([[{}]]);
  });
  it('returns a style object with the values', function () {
    mountWithContext( /*#__PURE__*/React.createElement(Component, {
      config: {
        color: {
          value: 'red'
        },
        paddingLeft: {
          value: 1
        }
      }
    }));
    expect(spy.mock.calls).toMatchObject([[{
      color: 'red',
      paddingLeft: 1
    }]]);
  });
  it('returns a style object with the responsive value', function () {
    mountWithContext( /*#__PURE__*/React.createElement(Component, {
      config: {
        color: {
          value: {
            base: 'red'
          }
        },
        paddingLeft: {
          value: {
            base: 1
          }
        }
      }
    }));
    expect(spy.mock.calls).toMatchObject([[{
      color: 'red',
      paddingLeft: 1
    }]]);
  });
  it('returns a style object with the transformed value', function () {
    mountWithContext( /*#__PURE__*/React.createElement(Component, {
      config: {
        color: {
          value: 'red',
          transform: function transform(value) {
            return "dark".concat(value);
          }
        },
        paddingLeft: {
          value: 1,
          transform: function transform(value) {
            return Number(value);
          }
        }
      }
    }));
    expect(spy.mock.calls).toMatchObject([[{
      color: 'darkred',
      paddingLeft: 1
    }]]);
  });
});